//
//    File: glmesh.h
//
//    (C) 2000-2008 Helmut Cantzler
//
//    Licensed under the terms of the Lesser General Public License.
//

#ifndef _OPENGL_FUNCTIONS_H
#define _OPENGL_FUNCTIONS_H

#include <GL/glut.h>
#include <GL/glu.h>
#include <stdio.h>

#ifndef WIN32
   #include <unistd.h>
#endif

#include "mesh.h"
#include "texture.h"

// Types

enum DisplayType {VERTICES=0, EDGES, TEXTURE, SOLID, WIRE,
		  FRONTLINES, FEATURES, NOTHING};
enum ActionType {NONE, TRANSLATE, ROTATE, ZOOM};

typedef struct
{
  Mesh *mesh, *features;
 
  // Viewpoint data (Trackball, Translation, Zoom, Clipping)
  GLfloat tbTransform[4][4];
  GLfloat xShift, yShift, zShift;
  GLfloat clipping;
 
  // Viewmode
  enum DisplayType meshDisplayMode, featuresDisplayMode;
 
  // Light intensity
  GLfloat lightBrightness;
 
  // Background colour
  GLfloat red, green, blue;

  // Flags
  bool displayNormals, displayShapeColors, displayFeatures, displayPolygons, displayBoundingBox;
  bool enableTextureFilter, enableCutBackFaces, enableLighting, enableAspectRatio;
} GLMeshSettings;

// Colours
enum colors {BLACK=0, BLUE, GREEN, RED, YELLOW, PURPLE, CYAN, WHITE};

const GLfloat ambientLightSource[] = {0.0, 0.0, 0.0, 1.0};

extern GLMeshSettings settings;

extern float surfaceNormalLength;
extern int numberOfShapes;
extern GLuint *textureList;
extern GLint windowWidth;
extern GLint windowHeight;
extern GLint windowX;
extern GLint windowY;
extern int lastMouseX, lastMouseY;
extern enum ActionType actionMode;
extern GLfloat tbLastPosition[3];
extern GLfloat tbAxis[3];
extern int translationZoom;

void draw(void);
void reshape(int width, int height);
void setTitle(void);
void key(unsigned char k, int x, int y);
void mouseButton(int button, int state, int x, int y);
void mouseDrag(int x, int y);
void mainMenu(int value);
void displayMenu(int value);
void openglInit(GLMeshSettings s);
void openglStart(void);

void checkForTraps(const char *string, int value);
void setMaterialColor(GLenum face, GLenum pname, int color);
void setMaterialColorGrey(GLenum face, GLenum pname, int b);
void displayNormals(void);
void displayBoundingBox(void);
void displayFeatures(void);
void displayPoints(list<Vertex*> *vertices);
void displayEdges(list<Edge*> *edges);
void displayMesh(void);
void displayMeshWithStencil(void);
void toggleTextureFilter(void);
void tbPointToVector(int x, int y, float v[3]);
void initLighting(void);
void initTexture(void);
void setShapeTexture(int j);

int width(void);
int height(void);

#endif
