//
//    File: main.cc
//
//    (C) 2000-2008 Helmut Cantzler
//
//    Licensed under the terms of the Lesser General Public License.
//

#include "mesh.h"
#include "list_mesh.h"
#include "p_mesh.h"
#include "gts_mesh.h"
#include "geomview_mesh.h"
#include "vrml1_mesh.h"
#include "vrml2_mesh.h"
#include "feature_mesh.h"
#include "shallo_mesh.h"

#include "glmesh.h"

int dummy1(int pos)
{
  return 0;
}

void dummy2(int size)
{
}

int main(int argc, char **argv)
{
  glutInit(&argc, argv);

  printf("\nMesh Viewer 0.3.2\n\n");

  if (argc != 2 && argc != 3)
    {
      printf("Syntax: %s <mesh file> <feature file>\n\n", argv[0]);
      exit(1);
    }

  GLMeshSettings settings;
  FILE *f;

  settings.mesh = new Mesh();
  settings.meshDisplayMode = SOLID;
  settings.featuresDisplayMode = NOTHING;
  settings.xShift = settings.yShift = settings.zShift = 0.0;
  settings.clipping = 0.0;
  settings.lightBrightness = 1.0;
  settings.red = settings.green = settings.blue = 0.0;
  settings.displayNormals = settings.displayShapeColors = settings.displayBoundingBox = false;
  settings.displayFeatures = false;   settings.displayPolygons = true;
  settings.enableTextureFilter = settings.enableCutBackFaces = false;
  settings.enableLighting = settings.enableAspectRatio = true;

  for (int i=0; i < 4; i++)
    for (int j=0; j < 4; j++)
      settings.tbTransform[i][j]= i == j ? 1.0 : 0.0;

  printf("Read mesh (%s) ...\n", argv[1]);
  if ((f = fopen(argv[1],"r")) == NULL)
    {
      fprintf(stderr, "Can't open %s for reading\n", argv[1]);
      exit(0);
    }

  switch (Mesh::type(f))
    {
    case LIST_MESH:
      settings.mesh = new ListMesh();
      break;
    case P_MESH:
      settings.mesh = new PMesh();
      break;
    case GTS_MESH:
      settings.mesh = new GTSMesh();
      break;
    case GEOMVIEW_MESH:
      settings.mesh = new GeomviewMesh();
      break;
    case VRML1_MESH:
      settings.mesh = new Vrml1Mesh();
      break;
    case VRML2_MESH:
      settings.mesh = new Vrml2Mesh();
      break;
    case SHALLO_MESH:
      settings.mesh = new ShalloMesh();
      break;
    case FEATURES_MESH:
      settings.mesh = new FeatureMesh();
      break;
    default:
      return 20;
    }

  settings.mesh->read(f, (int (*)(int)) dummy1, (void (*)(int)) dummy2);
  fclose(f);

  printf("Mesh: %d vertices and %d triangles\n",
	 settings.mesh->numberOfVertices(),
	 settings.mesh->numberOfTriangles());

  // Map vertices and triangle centroids to mesh centroid
  printf("Map mesh to centre ...\n");
  settings.mesh->moveToCentre();
  printf("Scale mesh ...\n");
  settings.mesh->scaleIntoNormalSphere();

  if (argc == 3)
    {
      printf("Read features ...\n");
      if ((f = fopen(argv[2],"r")) == NULL)
	{
	  fprintf(stderr, "Can't open %s for reading\n", argv[2]);
	  exit(0);
	}

      switch (Mesh::type(f))
	{
	case LIST_MESH:
	  settings.features = new ListMesh();
	  break;
	case P_MESH:
	  settings.features = new PMesh();
	  break;
	case GTS_MESH:
	  settings.features = new GTSMesh();
	  break;
	case GEOMVIEW_MESH:
	  settings.features = new GeomviewMesh();
	  break;
	case VRML1_MESH:
	  settings.features = new Vrml1Mesh();
	  break;
	case VRML2_MESH:
	  settings.features = new Vrml2Mesh();
	  break;
       case SHALLO_MESH:
         settings.features = new ShalloMesh();
	 break;
	case FEATURES_MESH:
	  settings.features = new FeatureMesh();
	  break;
	default:
	  return 20;
	}

      settings.features->read(f,  (int (*)(int)) dummy1, (void (*)(int)) dummy2);
      fclose(f);

      printf("Features: %d points and %d edges\n",
	     settings.features->numberOfVertices(),
	     settings.features->numberOfEdges());

      settings.features->scaleAccordingToReferenceMesh(settings.mesh);
    }
  else
    settings.features = new Mesh();

  openglInit(settings);

  printf("Start displaying ...\n");

  openglStart();
}
